package com.ff.apimngr.controller;


import com.ff.apimngr.entity.*;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.*;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 接口管理
 */
@RestController
@RequestMapping("/api/apimngr")
public class ApiController {

    @Resource
    private ApiService apiService;

    @Resource
    private ApiTypeService apiTypeService;

    @Resource
    private AppApiService appApiService;

    @Resource
    private UserService userService;

    @Resource
    private AuthService authService;

    @ApiOperation(value = "根据id获取接口", notes = "根据id获取接口")
    @GetMapping(value = "/findbyid")
    public Map<String, Object> getApiById(@ApiParam(value = "接口id", required = true) @RequestParam Integer id) {
        Map<String, Object> result = new HashMap<>();
        Api api = apiService.findApiById(id);
        if (null == api) {
            result.put("code", 10001);
            result.put("message", "未查询到此接口");
            return result;
        }
        result.put("data", api);
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }

    @ApiOperation(value = "根据姓名获取接口", notes = "根据姓名获取接口")
    @GetMapping(value = "/findbyname")
    public Map<String, Object> findApiByName(@ApiParam(value = "接口姓名", required = true) @RequestParam String name) {
        Map<String, Object> result = new HashMap<>();
        Api api = apiService.findApiByName(name);
        if (null == api) {
            result.put("code", 10001);
            result.put("message", "未查询到此接口");
            return result;
        }
        result.put("data", api);
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }

    @ApiOperation(value = "创建接口", notes = "创建接口")
    @PostMapping(value = "/insert")
    public Map<String, Object> insertApi(@ApiParam(value = "接口名字", required = true) @RequestParam String name,
                                         @ApiParam(value = "请求地址", required = true) @RequestParam String url,
                                         @ApiParam(value = "调用量限制", required = true) @RequestParam String callLimit,
                                         @ApiParam(value = "QPS限制", required = true) @RequestParam String qpsLimit,
                                         @ApiParam(value = "状态", required = true) @RequestParam Integer status,
                                         @ApiParam(value = "接口类型id", required = true) @RequestParam String apiTypeId,
                                         @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description,
                                         @ApiParam(value = "能力版本", required = false) @RequestParam(required = false) String version,
                                         @ApiParam(value = "开发文档", required = false) @RequestParam(required = false) String docUrl,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        if (!checkHasAdminAuth(request)) {
            result.put("code", 10001);
            result.put("message", "没有权限");
            return result;
        }
        Api api = apiService.findApiByName(name);
        if (null != api) {
            result.put("code", 10002);
            result.put("message", "接口名已存在");
            return result;
        }
        api = new Api();
        api.setName(name);
        api.setUrl(url);
        api.setCallLimit(callLimit);
        api.setQpsLimit(qpsLimit);
        api.setApiType(apiTypeService.findApiTypeById(Integer.valueOf(apiTypeId)));
        api.setStatus(status);//1=正常 0=不正常 2=测试（未发布） 3=VIP
        if (null == description) {
            api.setDescription("");
        } else {
            api.setDescription(description);
        }

        if (null == version) {
            api.setVersion("");
        } else {
            api.setVersion(version);
        }
        if (null == docUrl) {
            api.setDocUrl("");
        } else {
            api.setDocUrl(docUrl);
        }
        apiService.insertApi(api);

        result.put("code", 10000);
        result.put("message", "创建成功");
        return result;
    }

    @ApiOperation(value = "创建接口", notes = "创建接口")
    @PostMapping(value = "/insertNoAuth")
    public Map<String, Object> insertNoAuth(@ApiParam(value = "接口名字", required = true) @RequestParam String name,
                                         @ApiParam(value = "请求地址", required = true) @RequestParam String url,
                                         @ApiParam(value = "调用量限制", required = true) @RequestParam String callLimit,
                                         @ApiParam(value = "QPS限制", required = true) @RequestParam String qpsLimit,
                                         @ApiParam(value = "状态", required = true) @RequestParam Integer status,
                                         @ApiParam(value = "接口类型id", required = true) @RequestParam String apiTypeId,
                                         @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description,
                                         @ApiParam(value = "能力版本", required = false) @RequestParam(required = false) String version,
                                         @ApiParam(value = "开发文档", required = false) @RequestParam(required = false) String docUrl,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
//        if (!checkHasAdminAuth(request)) {
//            result.put("code", 10001);
//            result.put("message", "没有权限");
//            return result;
//        }
        Api api = apiService.findApiByName(name);
        if (null != api) {
            result.put("code", 10002);
            result.put("message", "接口名已存在");
            return result;
        }
        api = new Api();
        api.setName(name);
        api.setUrl(url);
        api.setCallLimit(callLimit);
        api.setQpsLimit(qpsLimit);
        api.setApiType(apiTypeService.findApiTypeById(Integer.valueOf(apiTypeId)));
        api.setStatus(status);//1=正常 0=不正常 2=测试（未发布） 3=VIP
        if (null == description) {
            api.setDescription("");
        } else {
            api.setDescription(description);
        }

        if (null == version) {
            api.setVersion("");
        } else {
            api.setVersion(version);
        }
        if (null == docUrl) {
            api.setDocUrl("");
        } else {
            api.setDocUrl(docUrl);
        }
        apiService.insertApi(api);

        result.put("code", 10000);
        result.put("message", "创建成功");
        return result;
    }



    @ApiOperation(value = "更新接口", notes = "更新接口")
    @PostMapping(value = "/update")
    public Map<String, Object> updateApi(@ApiParam(value = "接口id", required = true) @RequestParam String id,
                                         @ApiParam(value = "接口名字", required = true) @RequestParam String name,
                                         @ApiParam(value = "请求地址", required = true) @RequestParam String url,
                                         @ApiParam(value = "调用量限制", required = true) @RequestParam String callLimit,
                                         @ApiParam(value = "QPS限制", required = true) @RequestParam String qpsLimit,
                                         @ApiParam(value = "状态", required = true) @RequestParam Integer status,
                                         @ApiParam(value = "接口类型id", required = true) @RequestParam String apiTypeId,
                                         @ApiParam(value = "备注", required = false) @RequestParam(required = false) String description,
                                         @ApiParam(value = "能力版本", required = false) @RequestParam(required = false) String version,
                                         @ApiParam(value = "开发文档", required = false) @RequestParam(required = false) String docUrl,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        if (!checkHasAdminAuth(request)) {
            result.put("code", 10001);
            result.put("message", "没有权限");
            return result;
        }
        Api api = apiService.findApiById(Integer.valueOf(id));
        if (null == api) {
            result.put("code", 10001);
            result.put("message", "接口不存在");
            return result;
        }
        Api check = apiService.findApiByName(name);
        if (check != null && check.getId() != Integer.valueOf(id)) {
            result.put("code", 10001);
            result.put("message", "能力名称已存在,不可重复");
            return result;
        }
        api.setName(name);
        api.setUrl(url);
        api.setCallLimit(callLimit);
        api.setQpsLimit(qpsLimit);
        api.setStatus(status);
        api.setApiType(apiTypeService.findApiTypeById(Integer.valueOf(apiTypeId)));

        api.setDescription(description);
        api.setVersion(version);
        api.setDocUrl(docUrl);

        boolean rst = apiService.updateApi(api);
        result.put("code", 10000);
        result.put("message", "更新成功");
        return result;
    }


    @ApiOperation(value = "根据id删除接口", notes = "根据id删除接口")
    @PostMapping(value = "/delete")
    public Map<String, Object> deleteApi(@ApiParam(value = "接口id", required = true) @RequestParam String id,
                                         HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        if (!checkHasAdminAuth(request)) {
            result.put("code", 10001);
            result.put("message", "没有权限");
            return result;
        }
        Api api = apiService.findApiById(Integer.valueOf(id));
        if (null == api) {
            result.put("code", 10001);
            result.put("message", "删除失败，接口不存在");
            return result;
        }
        //检查接口是否在被应用使用
        List<AppApi> list = appApiService.listAppApisByApiId(Integer.valueOf(id));
        if (null != list && list.size() > 0) {
            result.put("code", 10002);
            result.put("message", "删除失败，有应用在使用此接口!");
            return result;
        }
        boolean rst = apiService.deleteApiById(Integer.valueOf(id));
        if (rst) {
            result.put("code", 10000);
            result.put("message", "删除成功");
        } else {
            result.put("code", 10002);
            result.put("message", "删除失败");
        }

        return result;
    }

    @ApiOperation(value = "根据id停用/启动接口", notes = "根据id停用/启动接口")
    @PostMapping(value = "/disable")
    public Map<String, Object> disableApi(@ApiParam(value = "接口id", required = true) @RequestParam String id,
                                          @ApiParam(value = "停用/启用接口", required = true) @RequestParam Integer status,
                                          HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        if (!checkHasAdminAuth(request)) {
            result.put("code", 10001);
            result.put("message", "没有权限");
            return result;
        }

        Api api = apiService.findApiById(Integer.valueOf(id));
        if (null == api) {
            result.put("code", 10001);
            result.put("message", "更新接口失败，接口不存在");
            return result;
        }
        api.setStatus(status);
        boolean rst = apiService.updateApi(api);
        if (rst) {
            result.put("code", 10000);
            result.put("message", "停用启用接口成功");
        } else {
            result.put("code", 10002);
            result.put("message", "停用启用接口失败");
        }

        return result;
    }

    @ApiOperation(value = "获取接口列表", notes = "获取接口列表")
    @GetMapping(value = "/list")
    public Map<String, Object> searchApis(@ApiParam(value = "搜索关键字", required = false) @RequestParam(required = false) String tag,
                                          @ApiParam(value = "接口状态 发布1/未发布0", required = false) @RequestParam(required = false) Integer status,
                                          @ApiParam(value = "能力角色id", required = false) @RequestParam(required = false) Integer roleId) {
        Map<String, Object> result = new HashMap<>();
        List<Api> list;
        if (null != roleId) {
            list = apiService.listApisByStatusAndRole(tag, status, roleId);
        } else {
            list = apiService.listApis(tag, status);
        }
        if (null != list) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询成功");
        }

        return result;
    }


    @ApiOperation(value = "获取接口列表 需要判断用户角色权限", notes = "获取接口列表")
    @GetMapping(value = "/listapis")
    public Map<String, Object> searchAllApis(HttpServletRequest request) {
        Map<String, Object> result = new HashMap<>();
        String token = request.getHeader("token");
        String account = JwtHelper.parseToken(token);
        Integer roleId = null;
        if (null != account && account.length() > 0) {
            User user = userService.findUserByAccount(account);
            if (null != user) {
                Role role = user.getRole();
                if (null != role) {
                    if (role.getRoleType() != 1) {
                        roleId = role.getId();
                    }

                }
            }
        }
        List<ApiType> types = apiTypeService.listApiTypes("");
        if (null == types) {
            result.put("code", 10001);
            result.put("message", "未配置接口类型，请联系管理员");
            return result;
        }
        List<Object> data = new ArrayList<>();

        List<String> apiTypeNames = new ArrayList<>();
        for (ApiType apiType : types) {
            Map<String, Object> map = new HashMap<>();
            List<Api> list = apiService.listApisByTypeAndRole(apiType.getId(), roleId);
            map.put("name", apiType.getName());
            if (null != list && list.size() > 0) {
                List<Object> apiList = new ArrayList<>();
                for (Api api : list) {
                    Map<String, Object> apiMap = new HashMap<>();
                    apiMap.put("label", api.getName());
                    apiMap.put("value", api.getId());
                    apiList.add(apiMap);
                }
                map.put("apis", apiList);
            }
            data.add(map);
            apiTypeNames.add(apiType.getName());
        }
        result.put("code", 10000);
        result.put("data", data);
        result.put("apiTypeNames", apiTypeNames);
        result.put("total", data.size());
        result.put("message", "获取接口列表成功");

        return result;
    }

    @ApiOperation(value = "获取应用数量统计", notes = "获取应用数量统计")
    @GetMapping(value = "/apicount")
    public Map<String, Object> searchDevelopersReport() throws ParseException {
        Map<String, Object> result = new HashMap<>();

        result.put("allApiNum", apiService.getAllApiNum());
        result.put("publishedNum", apiService.getPublishedApiNum());
        result.put("unPublishedNum", apiService.getUnPublishedApiNum());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String beginDate = format.format(new Date()) + " 00:00:00";
        Date beginDay = sdf.parse(beginDate);
        result.put("todayCallNum", apiService.getTodayCallNum(beginDay.getTime()));
        result.put("alarmApiNum", apiService.getAlarmApiNum());
        result.put("abnormalAppNum", apiService.getAbnormalApiNum());
        result.put("message", "查询完成");
        result.put("code", 10000);

        return result;
    }

    @ApiOperation(value = "获取能力调用排行榜", notes = "获取能力调用排行榜")
    @GetMapping(value = "/topapis")
    public Map<String, Object> searchTopDevelopers(@ApiParam(value = "开始日期", required = false) @RequestParam(required = false) String beginDate,
                                                   @ApiParam(value = "结束日期", required = false) @RequestParam(required = false) String endDate) throws ParseException {
        Map<String, Object> result = new HashMap<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        if (null == beginDate) {
            beginDate = format.format(new Date());
        }
        if (null == endDate) {
            endDate = format.format(new Date());
        }
        Date beginDay = sdf.parse(beginDate + " 00:00:00");
        Date endDay = sdf.parse(endDate + " 23:59:59");
        Long beginTime = beginDay.getTime();
        Long endTime = endDay.getTime();
        List<TopApi> list = apiService.getTopTenApis(beginDate, endDate);
        if (null != list && list.size() > 0) {
            int i = 1;
            List<TopApi> topApis = new ArrayList<>();
            for (TopApi topApi : list) {
                topApi.setId(i);
                float num = (float) topApi.getFailCount() / topApi.getTotalCount();
                DecimalFormat df = new DecimalFormat("0.00");
                topApi.setFailRate(Double.parseDouble(df.format(num)));
                i++;
                topApis.add(topApi);
            }
            result.put("data", topApis);
            result.put("total", list.size());
            result.put("code", 10000);
            result.put("message", "查询成功");
        } else {
            result.put("code", 10001);
            result.put("message", "没有查询到结果");
        }
        return result;
    }

    Boolean checkHasAdminAuth(HttpServletRequest request) {
        String token = request.getHeader("token");
        String account = "";
        if (token != null && token.length() > 0) {
            account = JwtHelper.parseToken(token);

            User user = userService.findUserByAccount(account);
            if (null != user) {
                Role role = user.getRole();
                if (role.getRoleType() == 1) {
                    return true;
                }
            }
        }
        return false;
    }

}
