package com.ff.apimngr.controller;


import com.ff.apimngr.entity.Api;
import com.ff.apimngr.entity.Auth;
import com.ff.apimngr.entity.Role;
import com.ff.apimngr.service.ApiService;
import com.ff.apimngr.service.AuthService;
import com.ff.apimngr.service.RoleService;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.ibatis.javassist.bytecode.stackmap.BasicBlock;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 角色管理
 */
@RestController
@RequestMapping("/api/role")
public class RoleController {

    @Resource
    private RoleService roleService;

    @Resource
    private ApiService apiService;

    @Resource
    private AuthService authService;

    @ApiOperation(value = "根据id获取角色", notes = "根据id获取角色")
    @GetMapping(value = "/findbyid")
    public Map<String, Object> getRoleById(@ApiParam(value = "角色id", required = true) @RequestParam Integer id) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleById(id);
        if (null == role) {
            result.put("code", 10001);
            result.put("message", "未查询到此角色");
            return result;
        }
        result.put("data", role);
        result.put("code", 10000);
        result.put("message", "获取成功");
        return result;
    }

    @ApiOperation(value = "根据姓名获取角色", notes = "根据姓名获取角色")
    @GetMapping(value = "/findbyname")
    public Map<String, Object> findRoleByName(@ApiParam(value = "角色姓名", required = true) @RequestParam String name) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleByName(name);
        if (null == role) {
            result.put("code", 10001);
            result.put("message", "未查询到此角色");
            return result;
        }
        result.put("data", role);
        result.put("code", 10000);
        result.put("message", "查询成功");
        return result;
    }

    @ApiOperation(value = "创建角色", notes = "创建角色")
    @PostMapping(value = "/create")
    public Map<String, Object> insertRole(@ApiParam(value = "角色名字", required = true) @RequestParam String roleName,
                                          //@ApiParam(value = "角色类型", required = true) @RequestParam Integer roleType,
                                          @ApiParam(value = "备注", required = false) @RequestParam String description) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleByName(roleName);
        if (null != role) {
            result.put("code", 10002);
            result.put("message", "角色名已存在");
            return result;
        }
        role = new Role();
        role.setRoleName(roleName);
        role.setRoleType(2);
        if(null == description) {
            role.setDescription("");
        } else {
            role.setDescription(description);
        }


        roleService.insertRole(role);

        result.put("code", 10000);
        result.put("message", "插入成功");
        return result;
    }

    @ApiOperation(value = "更新角色", notes = "更新角色")
    @PostMapping(value = "/update")
    public Map<String, Object> updateRole(@ApiParam(value = "角色id", required = true) @RequestParam String id,
                                          @ApiParam(value = "角色姓名", required = false) @RequestParam String roleName,
                                          //@ApiParam(value = "角色类型", required = false) @RequestParam Integer roleType,
                                          @ApiParam(value = "备注", required = false) @RequestParam String description) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleById(Integer.valueOf(id));
        if (null == role) {
            result.put("code", 10001);
            result.put("message", "角色不存在");
            return result;
        }
        if (!StringUtils.isEmpty(roleName)) {
            role.setRoleName(roleName);
        }
        /*if (null != roleType && roleType >0) {
            role.setRoleType(roleType);
        }*/

        role.setDescription(description);


        boolean rst = roleService.updateRole(role);
        result.put("code", 10000);
        result.put("message", "更新成功");
        return result;
    }


    @ApiOperation(value = "根据id删除角色", notes = "根据id删除角色")
    @PostMapping(value = "/delete")
    public Map<String, Object> deleteRole(@ApiParam(value = "角色id", required = true) @RequestParam String id) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleById(Integer.valueOf(id));
        if (null == role) {
            result.put("code", 10001);
            result.put("message", "删除失败，角色不存在");
            return result;
        }
        boolean rst = roleService.deleteRoleById(Integer.valueOf(id));
        if (rst) {
            result.put("code", 10000);
            result.put("message", "删除成功");
        } else {
            result.put("code", 10002);
            result.put("message", "删除失败");
        }

        return result;
    }

    @ApiOperation(value = "获取角色列表", notes = "获取角色列表")
    @GetMapping(value = "/list")
    public Map<String, Object> searchRoles(@ApiParam(value = "搜索关键字", required = false) @RequestParam(required = false) String tag) {
        Map<String, Object> result = new HashMap<>();
        List<Role> list = roleService.listRoles(tag);
        if (null != list) {
            result.put("data", list);
            result.put("total", list.size());
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询失败");
        }

        return result;
    }


    @ApiOperation(value = "根据角色id设置权限", notes = "根据角色id设置权限")
    @PostMapping(value = "/setauth")
    public Map<String, Object> setAuth(@ApiParam(value = "角色id", required = true) @RequestParam String roleId,
                                       @ApiParam(value = "角色id", required = true) @RequestParam String auths) {
        Map<String, Object> result = new HashMap<>();
        Role role = roleService.findRoleById(Integer.valueOf(roleId));
        if (null == role) {
            result.put("code", 10001);
            result.put("message", "设置权限失败，角色不存在");
            return result;
        }
        if (null != auths && auths.length() > 0) {
            String[] authArray = auths.split(",");
            //先删除旧权限数据
            authService.deleteAuthByRoleId(role.getId());
            for (String apiId : authArray) {
                try {
                    Api api = apiService.findApiById(Integer.valueOf(apiId));
                    Auth auth = new Auth();
                    auth.setApi(api);
                    auth.setRole(role);
                    authService.insertAuth(auth);
                } catch (Exception e) {
                    e.printStackTrace();
                    result.put("code", 10002);
                    result.put("message", "权限解析错误:" + e.getMessage());
                    return result;
                }
            }
        }
        result.put("code", 10000);
        result.put("message", "设置完成");
        return result;
    }


    @ApiOperation(value = "获取角色权限列表", notes = "获取角色权限列表")
    @GetMapping(value = "/getauth")
    public Map<String, Object> getAuth(@ApiParam(value = "角色id", required = true) @RequestParam String roleId) {
        Map<String, Object> result = new HashMap<>();
        List<Auth> list = authService.listByRoleId(Integer.valueOf(roleId));
        if (null != list) {
            List<Integer> auths = new ArrayList<>();
            for (Auth auth : list) {
                auths.add(auth.getApi().getId());
            }
            result.put("auths", auths);
            result.put("message", "查询完成");
            result.put("code", 10000);
        } else {
            result.put("code", 10001);
            result.put("total", 0);
            result.put("message", "查询失败");
        }

        return result;
    }
}
