package com.ff.apimngr.dao;

import com.ff.apimngr.entity.Api;
import com.ff.apimngr.entity.TopApi;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


@Mapper
public interface ApiMapper {


    /**
     * 根据id查找
     *
     * @param id
     * @return
     */
    Api findById(Integer id);

    /**
     * 根据姓名查找
     *
     * @param name
     * @return
     */
    Api findByName(String name);

    /**
     * 插入
     *
     * @param api
     * @return
     */
    boolean insert(Api api);

    /**
     * 更新
     *
     * @param api
     * @return
     */
    boolean update(Api api);

    /**
     * 根据id删除
     *
     * @param id
     * @return
     */
    boolean deleteById(Integer id);

    /**
     * 根据tag搜索列表
     *
     * @param tag
     * @return
     */
    List<Api> listApis(@Param("tag") String tag, Integer status);

    /**
     * 根据tag搜索列表
     *
     * @param tag
     * @return
     */
    List<Api> listApisByStatusAndRole(@Param("tag") String tag, @Param("status")Integer status, @Param("roleId") Integer roleId);

    /**
     * 根据apiTypeId搜索接口列表
     *
     * @param apiTypeId
     * @return
     */
    List<Api> listApisByType(@Param("apiTypeId") Integer apiTypeId);

    /**
     * 根据apiTypeId搜索接口列表
     *
     * @param roleId    用户角色id
     * @param apiTypeId
     * @return
     */
    List<Api> listApisByTypeAndRole(@Param("apiTypeId") Integer apiTypeId, @Param("roleId") Integer roleId);

    int getAllApiNum();
    int getPublishedApiNum();
    int getUnPublishedApiNum();
    int getTodayCallNum(@Param("startTime") Long startTime);
    int getAlarmApiNum();
    int getAbnormalApiNum();
    List<TopApi> getTopTenApis(@Param("startDate") String startDate, @Param("endDate") String endDate);

}
