package com.ff.apimngr.dao;

import com.ff.apimngr.entity.App;
import com.ff.apimngr.entity.TopApp;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


@Mapper
public interface AppMapper {


    /**
     * 根据id查找
     *
     * @param id
     * @return
     */
    App findById(Integer id);

    /**
     * 根据姓名查找
     *
     * @param name
     * @return
     */
    App findByName(String name);

    /**
     * 插入
     *
     * @param app
     * @return
     */
    Integer insert(App app);

    /**
     * 更新
     *
     * @param app
     * @return
     */
    boolean update(App app);

    /**
     * 根据id删除
     *
     * @param id
     * @return
     */
    boolean deleteById(Integer id);

    /**
     * 根据tag搜索列表
     *
     * @param tag
     * @return
     */
    List<App> listApps(@Param("tag") String tag, @Param("status") Integer status, @Param("userId") Integer userId, @Param("alarmStatus") Integer alarmStatus, @Param("expireDate") String expireDate);

    List<App> listAppsByExpireDate(@Param("expireDate") String expireDate);

    /**
     * 根据开发者已创建应用总数
     *
     * @param userId
     * @return
     */
    Integer listAppsCount(@Param("userId") Integer userId);

    /**
     * 根据开发者最近1个月创建应用总数
     *
     * @param userId
     * @return
     */
    Integer listRecentAppCount(@Param("userId") Integer userId);

    int getAllAppNum();

    int getTodayActiveAppNum(@Param("startTime") Long startTime);

    int getMonthActiveAppNum(@Param("startTime") Long startTime);

    int getAlarmAppNum();

    int getAbnormalAppNum();

    int getIllegalAppNum();

    List<TopApp> getTopTenApps(@Param("startDate") String startDate, @Param("endDate") String endDate);

    App findbyUserIdAndAppName(@Param("userId") Integer userId, @Param("appName") String appName);
}
